// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SHARED_CLIENTINFO_H
#define SHARED_CLIENTINFO_H

#include <set>

#include <QDataStream>
#include <QString>
#include <QCryptographicHash>

#include <shared/identity.h>

namespace Shared {

class ClientInfo
{
public:
    ClientInfo();

    QString getId() const;
    bool valid() const;

    QDataStream& operator << (QDataStream& stream);
    QDataStream& operator >> (QDataStream& stream) const;

public:
    std::set<Identity> identities;
    std::set<QString> extensions;
    QString node;
    QString verification;
    QString hash;
    QString specificPresence;

private:
    static const std::map<QString, QCryptographicHash::Algorithm> hashes;
};

}

QDataStream& operator << (QDataStream& stream, const Shared::ClientInfo& info);
QDataStream& operator >> (QDataStream& stream, Shared::ClientInfo& info);

#endif // SHARED_CLIENTINFO_H
